@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -34,6 +34,7 @@
 #include "pango-engine-private.h"
 #include "pango-fontmap.h"
 #include "pango-impl-utils.h"
+#include "modules.h"
 
 #undef PANGO_DISABLE_DEPRECATED
 
@@ -232,7 +233,7 @@ static GSList *registered_displays;
 
 static int
 close_display_cb (Display   *display,
-		  XExtCodes *extcodes)
+		  XExtCodes *extcodes G_GNUC_UNUSED)
 {
   pango_x_shutdown_display (display);
   registered_displays = g_slist_remove (registered_displays, display);
@@ -278,9 +279,18 @@ pango_x_font_map_for_display (Display *display)
   char **xfontnames;
   int num_fonts, i;
   int screen;
+  static gboolean registered_modules = FALSE;
 
   g_return_val_if_fail (display != NULL, NULL);
 
+  if (!registered_modules)
+    {
+      registered_modules = TRUE;
+
+      for (i = 0; _pango_included_x_modules[i].list; i++)
+	pango_module_register (&_pango_included_x_modules[i]);
+    }
+
   /* Make sure that the type system is initialized */
   g_type_init ();
 
@@ -387,7 +397,7 @@ pango_x_font_map_finalize (GObject *object)
 }
 
 static void
-list_families_foreach (gpointer key,
+list_families_foreach (gpointer key G_GNUC_UNUSED,
 		       gpointer value,
 		       gpointer user_data)
 {
@@ -449,7 +459,7 @@ pango_x_get_font_family (PangoXFontMap *xfontmap,
 
 static PangoFont *
 pango_x_font_map_load_font (PangoFontMap               *fontmap,
-			    PangoContext               *context,
+			    PangoContext               *context G_GNUC_UNUSED,
 			    const PangoFontDescription *description)
 {
   PangoXFontMap *xfontmap = (PangoXFontMap *)fontmap;
@@ -509,7 +519,6 @@ pango_x_font_map_load_font (PangoFontMap               *fontmap,
 	    {
 	      PangoXFont *xfont = pango_x_font_new (fontmap, best_match->xlfd, size);
 
-	      xfont->fontmap = fontmap;
 	      xfont->xface = best_match;
 	      best_match->cached_fonts = g_slist_prepend (best_match->cached_fonts, xfont);
 
@@ -536,8 +545,8 @@ pango_x_font_map_load_font (PangoFontMap               *fontmap,
 static gboolean error_occurred;
 
 static int
-ignore_error (Display     *d,
-	      XErrorEvent *e)
+ignore_error (Display     *d G_GNUC_UNUSED,
+	      XErrorEvent *e G_GNUC_UNUSED)
 {
   return 0;
 }
@@ -1634,7 +1643,7 @@ pango_x_family_list_faces (PangoFontFamily  *family,
     }
 }
 
-G_CONST_RETURN char *
+static G_CONST_RETURN char *
 pango_x_family_get_name (PangoFontFamily  *family)
 {
   PangoXFamily *xfamily = PANGO_X_FAMILY (family);
